#include <Tone.h>
#include <Peggy2.h>
#include <avr/pgmspace.h>
#include "Characters.h"

// Frame split into 3 sub-frames (for 8 brightness levels)
Peggy2 frame4;
Peggy2 frame2;
Peggy2 frame1;

Peggy2 walls;
Peggy2 apples;

Tone notePlayer;

// Define game area
const byte gameMinX = 0;
const byte gameMaxX = 24;
const byte gameMinY = 0;
const byte gameMaxY = 23;
const byte gameCentreX = (gameMaxX-gameMinX)/2;
const byte gameCentreY = (gameMaxY-gameMinY)/2;

// Define score area
const byte scoreMinX = 0;
const byte scoreMaxX = 24;
const byte scoreY = 24;

#define SNAKE 0
#define PONG 1
#define BREAKOUT 2
#define RACE 3
#define GAMES 4
#define SCORE 4
#define LEVEL 5

#define NONE 0xFF
byte gameType = NONE;
prog_char string_0[] PROGMEM = "SNAKE";
prog_char string_1[] PROGMEM = "PONG";
prog_char string_2[] PROGMEM = "BREAK";
prog_char string_3[] PROGMEM = "RACE";
prog_char string_4[] PROGMEM = "SCORE";
prog_char string_5[] PROGMEM = "LEVEL";
PROGMEM const char *stringTable[] =
{   
  string_0,
  string_1,
  string_2,
  string_3,
  string_4,
  string_5 };
  

int points=0;
byte level=1;
byte levelMax=25; // Score to go to next level
byte speed=30;


void setup()
{
  frame4.HardwareInit();   // Call this once to init the hardware. 

  randomSeed(analogRead(5));
  notePlayer.begin(19);
  
  // Setup buttons
  DDRB = 0xFE;    // PORTB pin 0 is button 0, set it up as an input
  PORTB |= 0x01;  // PORTB pin 0 pull up
  DDRC = 0xE0;    // PORTC pins 0 - 4 set up as inputs corresponding to buttons 1 - 5
  PORTC |= 0x1F;  // PORTC pins 0 - 4 pull ups
}

void loop()
{
  if (gameType == SNAKE) {
    snake_loop();
  } 
  else if (gameType==PONG) {
    pong_loop();
  } else if (gameType==BREAKOUT) {
    breakout_loop();
  } else if (gameType==RACE) {
    race_loop();
  } 
  else {
    display_intro();
  }
}

void setup_game()
{
  if (gameType == SNAKE) {
    snake_setup_game();
  } else if (gameType == PONG) {
    pong_setup_game();
  } else if (gameType == BREAKOUT) {
    breakout_setup_game();
  } else if (gameType == RACE) {
    race_setup_game();
  } else {
    display_intro();
  }
}


void add_wall(byte x1, byte y1,byte x2, byte y2)
{
  setLine(x1,y1,x2,y2,7);
  walls.Line(x1,y1,x2,y2);
}

void update_scorebar()
{
  byte mypoints = points % (levelMax);
  for (byte i=scoreMinX;i<=scoreMaxX;i++) {
    if ((i-scoreMinX) < mypoints) {
      setPixel(i,scoreY,7);
    } 
    else {
      setPixel(i,scoreY,0);
    }
  }  
}

void display_score()
{
  clear_display();

  write_string("SCORE",0,0,7);
  write_number(points,0,7,7);

  write_string("LEVEL",1,18,7);
  write_number(level,16,11,7);

  refresh_display(50);

  while((PINC & 0x1F) == 0x1F) {  // Wait for button 1-5
    refresh_display(1);
  }
}

void display_intro()
{
  clear_display();
  byte selectDir=0;
  gameType=0;
  write_predef((gameType+GAMES-1) % GAMES,1,1,3);
  write_predef(gameType,1,9,7);
  write_predef((gameType+1) % GAMES,1,17,3);
  
  while ((PINC & 0x01) && (PINB & 0x1)) {  // Wait for button 1 or select button
    playMelody();
    selectDir=0;
    if (!(PINC & 0x04)) selectDir=1; // UP
    if (!(PINC & 0x08)) selectDir=2; // DOWN
    if (selectDir) {
      if (selectDir==1) {
        gameType=(gameType+1) % GAMES;
      } else {
         gameType=(gameType+GAMES-1) % GAMES;
      }
      for (byte k=0; k<8 ; k++) {
        playMelody();
        clear_display();
        if(selectDir == 1) {
          byte i=8-k;
          write_predef((gameType+GAMES-2) % GAMES,1,-7+i,3);
          write_predef((gameType+GAMES-1) % GAMES,1,1+i,3);
          write_predef(gameType,1,9+i,3);
          write_predef((gameType+1) % GAMES,1,17+i,3);
        } else {
          write_predef((gameType+GAMES-1) % GAMES,1,-7+k,3);
          write_predef(gameType,1,1+k,3);
          write_predef((gameType+1) % GAMES,1,9+k,3);
          write_predef((gameType+2) % GAMES,1,17+k,3);

        }
        refresh_display(5);
      }
     
      clear_display();
      write_predef((gameType+GAMES-1) % GAMES,1,1,3);
      write_predef(gameType,1,9,7);
      write_predef((gameType+1) % GAMES,1,17,3);
    }
    refresh_display(1);
  }
  notePlayer.stop();
  setup_game();

}

void setPixel(byte x, byte y, byte brightness)
{
  if (x>24 || y>24) return;
  frame4.WritePoint(x,y,(brightness & 4));
  frame2.WritePoint(x,y,(brightness & 2));
  frame1.WritePoint(x,y,(brightness & 1));
}

void setLine(byte x1, byte y1, byte x2, byte y2, byte brightness)
{
  if (brightness & 4) frame4.Line(x1,y1,x2,y2);
  if (brightness & 2) frame2.Line(x1,y1,x2,y2);
  if (brightness & 1) frame1.Line(x1,y1,x2,y2);
}

void refresh_display (byte repeat)
{
  for (byte i=0; i<repeat; i++) {
    frame4.RefreshAll(4);
    frame2.RefreshAll(2);
    frame1.RefreshAll(1);
  }
}

void clear_display()
{
  frame1.Clear();
  frame2.Clear();
  frame4.Clear();
}

void write_number(int num, int x, int y, byte brightness)
{
  char numChar[10];
  itoa(num, numChar, 10);
  write_string(numChar,x,y,brightness);
}

void write_string(char string[], byte x, byte y, byte brightness)
{
  byte i=0;
  while (string[i] != '\0') {
    show_char(string[i], x+(5*i), y, brightness);
    i++;
  }
}

void write_predef(byte stringNumber, byte x, byte y, byte brightness)
{
  char buffer[10];
  strcpy_P(buffer, (char*)pgm_read_word(&(stringTable[stringNumber])));
  write_string(buffer,x,y,brightness);
}

void show_char(char Character, byte x, byte y, byte brightness)
{
  byte charNum = 36;

  if ((Character >= '0') && (Character <= '9')) {
    charNum=Character-48;
  }
  if ((Character >= 'A') && (Character <= 'Z')) {
    charNum=Character-55;
  }
  
  unsigned long w;
  w = pgm_read_dword_near(chars + charNum);
  for (byte i=0; i<6; i++) {
    byte offset = 4*(5-i);
    byte b = w >> offset; // get four bits
    b=b & 0xF;
    if (b & B1000) setPixel(x,y+i,brightness);
    if (b & B0100) setPixel(x+1,y+i,brightness);
    if (b & B0010) setPixel(x+2,y+i,brightness);
    if (b & B0001) setPixel(x+3,y+i,brightness);
  }
}
