// Define snake properties;
const byte snakeMaxLength = 128;
byte snakeX[snakeMaxLength];
byte snakeY[snakeMaxLength];
byte snakeHead;
byte snakeLength;
int snakeDirX;
int snakeDirY;

void snake_loop()
{

  int newSnakeDirX=0;
  int newSnakeDirY=0;
  if (!(PINC & 0x02)) newSnakeDirX=-1;  // PORT C pin 1 is attached to left button
  if (!(PINC & 0x04)) newSnakeDirY=+1;  // PORT C pin 2 is attached to down button
  if (!(PINC & 0x08)) newSnakeDirY=-1;  // PORT C pin 3 is attached to up button
  if (!(PINC & 0x10)) newSnakeDirX=+1;  // PORT C pin 4 is attached to right button

  if (newSnakeDirX || newSnakeDirY) {
    if (!((snakeDirX == -newSnakeDirX) && (snakeDirY == -newSnakeDirY))) {
      snakeDirX=newSnakeDirX;
      snakeDirY=newSnakeDirY;
    }
  }

  snake_move();
  refresh_display(speed);
}

void snake_add_apple()
{
  byte appleX = random(gameMinX+1,gameMaxX);
  byte appleY = random(gameMinY+1,gameMaxY);
  if (frame1.GetPoint(appleX,appleY) || frame2.GetPoint(appleX,appleY) || frame4.GetPoint(appleX,appleY)) {
    snake_add_apple();
    return;
  }

  apples.SetPoint(appleX,appleY);
  setPixel(appleX,appleY,4);
}

void snake_setup_snake(byte x, byte y)
{
  for (byte i=0;i<snakeMaxLength;i++) {
    snakeX[i]=0xFF; // marker for unused spot
    snakeY[i]=0xFF; // marker for unused spot
  }
  snakeLength=1;
  snakeHead=snakeLength-1;
  snakeX[snakeHead] = x,
  snakeY[snakeHead] = y,
  setPixel(snakeX[snakeHead],snakeY[snakeHead],7);
  snakeDirX=random(3)-1;
  snakeDirY=random(3)-1;
  if ((snakeDirX==0) && (snakeDirY==0)) snakeDirX=1;
}

void snake_move ()
{
  byte oldsnakeHead = snakeHead;
  byte oldsnakeTail=(snakeMaxLength+snakeHead-snakeLength) % snakeMaxLength;
  snakeHead=(snakeHead+1) % snakeMaxLength;

  byte snakeTail=(oldsnakeTail +1) % snakeMaxLength;

  snakeX[snakeHead] = (snakeX[oldsnakeHead] + snakeDirX) % (gameMaxX+1);
  if ((snakeX[snakeHead] < gameMinX) || (snakeX[snakeHead] > 50)) snakeX[snakeHead] = gameMaxX;

  snakeY[snakeHead] = (snakeY[oldsnakeHead] + snakeDirY) % (gameMaxY+1);
  if ((snakeY[snakeHead] < gameMinY) || (snakeY[snakeHead] > 50)) snakeY[snakeHead] = gameMaxY;

  // Check if hitting an apple
  if (apples.GetPoint(snakeX[snakeHead],snakeY[snakeHead])) {
    points++;

    if ((points % levelMax) == 0) {
      //next level
      level++;
      speed = 32-((points % levelMax)/2)-(2*level);
      display_score();
      snake_setup_level();
      return;
    }
    speed = 32-((points % levelMax)/2)-(2*level);
    snakeLength=snakeLength+2;
    apples.ClearPoint(snakeX[snakeHead],snakeY[snakeHead]);
    snake_add_apple();
    update_scorebar();
  }

  boolean hit = false;
  // Check if hiiting self
  for (byte i=0; i< snakeMaxLength; i++) {
    if (i != snakeHead) {
      if ((snakeX[i] == snakeX[snakeHead]) && (snakeY[i] == snakeY[snakeHead])) {
        hit=true;
      }
    }
  }
  // Check if hitting wall
  if (walls.GetPoint(snakeX[snakeHead],snakeY[snakeHead])) {
    hit=true;
  }

  if (hit) {
    display_score();
    snake_setup_game();
  } 
  else {
    // Set the head/tail values
    setPixel(snakeX[oldsnakeTail],snakeY[oldsnakeTail],0);
    snakeX[oldsnakeTail] = 0xFF;
    snakeY[oldsnakeTail] = 0xFF;
    setPixel(snakeX[snakeTail],snakeY[snakeTail],3);
    setPixel(snakeX[snakeHead],snakeY[snakeHead],7);
  }
}

void snake_setup_board()
{

  byte numLevels = 4;
  byte myLevel=((level-1) % numLevels) + 1;
  walls.Clear();
  if (myLevel == 1) {
    // Four outside walls
    add_wall(gameMinX,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameMaxX,gameMaxY);
    snake_setup_snake(gameCentreX, gameCentreY);
  } 
  else if (myLevel == 2) {
    // Four outside walls plus internal bar
    add_wall(gameMinX,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameMaxX,gameMaxY);
    add_wall((gameMaxX/3), gameCentreY, (gameMaxX/3*2), gameCentreY);
    snake_setup_snake(gameCentreX, (gameMaxY-gameMinY)/3);
  } 
  else if (myLevel == 3) {
    // Four outside walls with gaps plus internal cross
    add_wall(gameMinX,gameMinY,gameCentreX-3,gameMinY);
    add_wall(gameCentreX+3,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameCentreY-3);
    add_wall(gameMaxX,gameCentreY+3,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameCentreY-3);
    add_wall(gameMinX,gameCentreY+3,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameCentreX-3,gameMaxY);
    add_wall(gameCentreX+3,gameMaxY,gameMaxX,gameMaxY);
    add_wall((gameMaxX/3), gameCentreY, (gameMaxX/3*2), gameCentreY);
    add_wall(gameCentreX, gameMaxY/3, gameCentreX, gameMaxY/3*2);
    snake_setup_snake(gameCentreX, gameCentreY-5);
  } 
  else if (myLevel == 4) {
    // Two walls splitting board into quarters
    add_wall(gameCentreX, gameMinY, gameCentreX, gameMaxX);
    add_wall(gameMinX, gameCentreY, gameMaxX, gameCentreY);
    snake_setup_snake(gameMinX+2, gameMinY+2);
  }
}

void snake_setup_game()
{
  points=0;
  level=1;
  clear_display();
  write_predef(LEVEL,1,1,7);
  write_number(level,8,12,7);
  int oldlevel = level;
  refresh_display(100);

  while ((PINC & 0x01) && (PINB & 0x1)) {  // Wait for button 1-5
    if (!(PINC & 0x04)) level--;
    if (!(PINC & 0x08)) level++;
    if (level==0) level=1;
    if (level != oldlevel) {
      clear_display();
      write_predef(LEVEL,1,1,7);
      write_number(level,8,12,7);
      refresh_display(50);
      oldlevel=level;
    }
    refresh_display(1);
  }
  snake_setup_level();
}

void snake_setup_level()
{
  clear_display();

  // 5 initial apples
  apples.Clear();
  for (byte i=0; i<5; i++) {
    snake_add_apple();
  }

  snake_setup_board();  
}
